﻿using CyUSB;
using DataAcquisitionCardDemo.Config;
using Finetooling.DAL;
using DataAcquisitionCardDemo.Model;
using OxyPlot;
using OxyPlot.Annotations;
using OxyPlot.Axes;
using OxyPlot.Series;
using OxyPlot.WindowsForms;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Finetooling.DAL.Model;

namespace DataAcquisitionCardDemo.ADC
{
    public partial class ADCForm : Form
    {
        private DAQBuffer daqBuffer = new DAQBuffer();

        private EBSData ebsData = new EBSData();

        private PulseData pulseData = new PulseData();

        private PlotView pv1 = new PlotView();
        private PlotView pv2 = new PlotView();

        bool IsContinueRead = false;
        ADCConfig adc = new ADCConfig();

        //该线程用于不断接受DAQ的数据，用于ADC采集
        Thread DataReceiveThread;
        //该线程用以处理ADC采集数据，用以展示数据
        Thread DataHandlerThread;

        float signalLevel = 1;
        float triggerLevel = 4;

        private List<float> TriggerWave1 = new List<float>();
        private List<float> SignalWave1 = new List<float>();
        private List<float> TriggerWave2 = new List<float>();
        private List<float> SignalWave2 = new List<float>();

        /// <summary>
        /// 传感器的脉宽结果
        /// </summary>
        SensorPulse sensor1Pulse = new SensorPulse();
        SensorPulse sensor2Pulse = new SensorPulse();

        private double DtDist1;
        private double DtDist2;

        private LineSeries lineASignal = new LineSeries(); // yello
        private LineSeries lineATri1 = new LineSeries(); // blue
        private LineSeries lineATri2 = new LineSeries(); // green
        private LineSeries lineBSignal = new LineSeries(); // yellow
        private LineSeries lineBTri1 = new LineSeries(); // blue
        private LineSeries lineBTri2 = new LineSeries(); // green

        private LineAnnotation lineA11 = new LineAnnotation();
        private LineAnnotation lineA12 = new LineAnnotation();
        private LineAnnotation lineA21 = new LineAnnotation();
        private LineAnnotation lineA22 = new LineAnnotation();
        private LineAnnotation lineA31 = new LineAnnotation();
        private LineAnnotation lineA32 = new LineAnnotation();
        private LineAnnotation lineA41 = new LineAnnotation();
        private LineAnnotation lineA42 = new LineAnnotation();

        private LineAnnotation lineB11 = new LineAnnotation();
        private LineAnnotation lineB12 = new LineAnnotation();
        private LineAnnotation lineB21 = new LineAnnotation();
        private LineAnnotation lineB22 = new LineAnnotation();
        private LineAnnotation lineB31 = new LineAnnotation();
        private LineAnnotation lineB32 = new LineAnnotation();
        private LineAnnotation lineB41 = new LineAnnotation();
        private LineAnnotation lineB42 = new LineAnnotation();

        private static ConfigParams configs = MainForm.configs;

        public ADCForm()
        {
            InitializeComponent();
        }

        private void ADCForm_Load(object sender, EventArgs e)
        {
            adc.Channel = 1;
            adc.ChannelMode = ADCChannelMode.Single;
            adc.Resolution = 16;
            adc.SampleMode = ADCSampleMode.Continue;
            adc.SampleRate = 800000;
            adc.SignalType = ADCSignelType.Volt;
            adc.Amplitude = 10;
            adc.Length = 8000;

            Init();
        }

        void Init()
        {
            InitOxyPlot();
            ClearDgv();

            //txtSignalLevel.Text = signalLevel.ToString();
            //txtTriggerLevel.Text = triggerLevel.ToString();
        }
        
        public void UpdateDevice(List<string> list)
        {
            cmbDev.Items.Clear();
            for (int i = 0; i < list.Count; i++)
            {
                cmbDev.Items.Add(list[i]);
            }

            if (cmbDev.Items.Count > 0)
            {
                cmbDev.SelectedIndex = 0;
            }
            else
            {
                cmbDev.Text = "";
            }
        }

        private delegate void DlgtUpdateTestResult(TestResultEnum testResult);

        private void UpdateTestResult(TestResultEnum testResult)
        {
            if (testResult == TestResultEnum.Pass)
            {
                lalTestResult.Text = "PASS";
                lalTestResult.BackColor = Color.LightGreen;
            }
            else if (testResult == TestResultEnum.Fail)
            {
                lalTestResult.Text = "Fail";
                lalTestResult.BackColor = Color.OrangeRed;
            }
            else
            {
                lalTestResult.Text = "IDLE";
                lalTestResult.BackColor = Color.White;
            }
        }

        private delegate void DlgtUpdatePlot();

        private void UpdatePlot()
        {
            double step = 1000.0 / (adc.SampleRate / 4);

            lineASignal.Points.Clear();
            lineATri1.Points.Clear();
            lineATri2.Points.Clear();
            lineBSignal.Points.Clear();
            lineBTri1.Points.Clear();
            lineBTri2.Points.Clear();

            int len = 0;
            if (TriggerWave1 != null && TriggerWave1.Count > 0)
            {
                len = TriggerWave1.Count;
            }
            else if (TriggerWave2 != null && TriggerWave2.Count > 0)
            {
                len = TriggerWave2.Count;
            }

            for (int i = 0; i < len; i++)
            {
                double x = i * step;
                if (TriggerWave1 != null && TriggerWave1.Count > i)
                {
                    lineATri1.Points.Add(new DataPoint(x, TriggerWave1[i]));
                    lineBTri1.Points.Add(new DataPoint(x, TriggerWave1[i]));
                }
                if (SignalWave1 != null && SignalWave1.Count > i)
                {
                    lineASignal.Points.Add(new DataPoint(x, SignalWave1[i]));
                }

                if (TriggerWave2 != null && TriggerWave2.Count > i)
                {
                    lineATri2.Points.Add(new DataPoint(x, TriggerWave2[i]));
                    lineBTri2.Points.Add(new DataPoint(x, TriggerWave2[i]));
                }
                if (SignalWave2 != null && SignalWave2.Count > i)
                {
                    lineBSignal.Points.Add(new DataPoint(x, SignalWave2[i]));
                }
            }

            UpdateXAxis(0,step * len);

            //更新脉冲宽度显示值
            UpdateDgv();

            pv1.Refresh();
            pv2.Refresh();
        }

        /// <summary>
        /// 每次采集的字节数19200，也就是9600个点，采集时间约12ms
        /// </summary>
        private const int packetReceiveSize = 19200;
        /// <summary>
        /// 将用户输入的参数设置到变量中
        /// </summary>
        private void SetConfigParam()
        {
            signalLevel = configs.SignalLevel;
            triggerLevel = configs.TriggerLevel;
        }

        private void tsbADCStart_Click(object sender, EventArgs e)
        {
            if (tsbADCStart.Text == "开始")
            {
                ADCStart();
            }
            else
            {
                ADCStop();
            }
        }

        private void BeginADCStart()
        {
            //将用户输入的参数设置到变量中
            SetConfigParam();

            //重新设置参数
            UpdateXAxis(0, 3000);
            daqBuffer.Clear();
            ebsData.Clear();

            pulseData.SetParam(signalLevel,adc.SampleRate);

            TriggerWave1.Clear();
            TriggerWave2.Clear();
            SignalWave1.Clear();
            SignalWave2.Clear();
            lineA11.X = 0;
            lineA12.X = 0;
            lineA21.X = 0;
            lineA22.X = 0;
            lineA31.X = 0;
            lineA32.X = 0;
            lineA41.X = 0;
            lineA42.X = 0;

            lineB11.X = 0;
            lineB12.X = 0;
            lineB21.X = 0;
            lineB22.X = 0;
            lineB31.X = 0;
            lineB32.X = 0;
            lineB41.X = 0;
            lineB42.X = 0;

            DtDist1 = 0;
            DtDist2 = 0;
            sensor1Pulse.Clear();
            sensor2Pulse.Clear();

            Invoke((DlgtUpdatePlot)UpdatePlot);
            Invoke((DlgtUpdateTestResult)UpdateTestResult, TestResultEnum.Idle);
        }

        private void ADCStart()
        {
            try
            {
                //开始采集前的设置
                BeginADCStart();

                double dSampleInterval = 1.0 / adc.SampleRate * 50 * 1000000;
                int nSampleInterval = Convert.ToInt32(dSampleInterval);
                int state = 0;
                CyUSBDevice dev = MainForm.daqs[adc.Device].dev;
                DAQHelper.ADCStopClearBuffer(dev);
                state = DAQHelper.SetADCChannelMode(dev, adc.ChannelMode);
                state = DAQHelper.SetADCSampleMode(dev, ADCSampleMode.Continue);
                state = DAQHelper.SetADCSampleRate(dev, nSampleInterval);
                state = DAQHelper.SetADCSignalType(dev, adc.SignalType);
                state = DAQHelper.SetADCRange(dev, MainForm.daqs[adc.Device].Limit.dictADCRange[adc.Amplitude.ToString()]);
                //采样通道：0,1,2,3
                int ch = 0;
                ch |= 0x01 << 0;
                ch |= 0x01 << 1;
                ch |= 0x01 << 2;
                ch |= 0x01 << 3;
                state = DAQHelper.SetADCSampleChannel(dev, ch);

                if (state == 0)
                {
                    IsContinueRead = true;
                    //数据采集线程
                    DataReceiveThread = new Thread(ReceiveADCData);
                    DataReceiveThread.IsBackground = true;
                    DataReceiveThread.Priority = ThreadPriority.AboveNormal;
                    DataReceiveThread.Start();

                    //数据分析线程
                    DataHandlerThread = new Thread(HandlerADCData);
                    DataHandlerThread.IsBackground = true;
                    DataHandlerThread.Priority = ThreadPriority.AboveNormal;
                    DataHandlerThread.Start();

                    UpdateControlState(false);
                    tsbADCStart.Text = "停止";
                }
            }
            catch (Exception ex)
            {
                IsContinueRead = false;
                MessageBox.Show(ex.Message);
            }
        }

        private void ADCStop()
        {
            IsContinueRead = false;
            UpdateControlState(true);
            tsbADCStart.Text = "开始";
        }

        #region ADC接收数据并展示波形

        private void ReceiveADCData()
        {
            byte[] packetBuf = new byte[packetReceiveSize];

            CyUSBDevice dev = MainForm.daqs[adc.Device].dev;
            dev.BulkInEndPt.TimeOut = 1000;  //设置100ms的超时时间
            DAQHelper.SetADCStart(dev, Startup.Start);

            while (IsContinueRead)
            {
                //缓存到队列的形式
                int state = DAQHelper.ReceivePacketData(dev, ref packetBuf, packetReceiveSize);
                if (state == DAQInfo.SUCCESS)
                {
                    //将读取到的数据放到framebuffer
                    if (!daqBuffer.Add(packetBuf))
                    {
                        ADCStop();
                        MessageBox.Show("error!\nreceive buffer is overflowed!");
                    }
                }
            }
            DAQHelper.ADCStopClearBuffer(dev);
        }

        private int findPulseCount = 10;
        private float FindMinPulse(ref List<float> pulseList)
        {
            if (pulseList == null || pulseList.Count <= 0)
            {
                return 0;
            }

            int startIndex = (pulseList.Count / 2) - (findPulseCount / 2);
            if (startIndex < 0)
            {
                startIndex = 0;
            }
            int endIndex = startIndex + findPulseCount;
            if(endIndex > pulseList.Count)
            {
                endIndex = pulseList.Count;
            }

            float min = 0;
            for (int i = startIndex; i < endIndex; i++)
            {
                min += pulseList[i];
            }
            min /= (endIndex - startIndex);
            min = (float)Math.Round(min);
            return min;
        }

        private float FindMaxPulse(ref List<float> pulseList)
        {
            if(pulseList==null || pulseList.Count <= 0)
            {
                return 0;
            }

            int startIndex = (pulseList.Count / 2) - (findPulseCount / 2);
            if (startIndex < 0)
            {
                startIndex = 0;
            }
            int endIndex = startIndex + findPulseCount;
            if (endIndex > pulseList.Count)
            {
                endIndex = pulseList.Count;
            }

            float max = 0;
            for (int i = startIndex; i < endIndex; i++)
            {
                max += pulseList[i];
            }

            max /= (endIndex - startIndex);
            max = (float)Math.Round(max);
            return max;
        }

        private void HandlerADCData()
        {
            while (IsContinueRead)
            {
                //这里处理之前需要多缓存一包数据
                if(daqBuffer.Length <= packetReceiveSize)
                {
                    continue;
                }

                int bufLen = packetReceiveSize;
                byte[] tmpBuf = new byte[packetReceiveSize];
                daqBuffer.Get(ref bufLen, ref tmpBuf);
                if (bufLen == packetReceiveSize)
                {
                    DAQData data = new DAQData(tmpBuf, bufLen, (float)adc.Amplitude);

                    //将原始数据转为波形数据
                    EBSData tmpEbs = new EBSData();
                    tmpEbs.AddLast(data);
                    float[] triggerData1 = tmpEbs.GetTriggerData1();
                    float[] triggerData2 = tmpEbs.GetTriggerData2();
                    float[] signalData1 = tmpEbs.GetSignalData1();
                    float[] signalData2 = tmpEbs.GetSignalData2();

                    TriggerWave1.AddRange(triggerData1);
                    TriggerWave2.AddRange(triggerData2);
                    SignalWave1.AddRange(signalData1);
                    SignalWave2.AddRange(signalData2);

                    //超过20s没有读取到下降沿信号，则停止采集
                    if (TriggerWave1.Count > 4000000)
                    {
                        ADCStop();
                        MessageBox.Show("超过20s未采集到SW2的低电平触发信号，停止采集");
                        return;
                    }

                    //判断是否有触发信号
                    EBS ebs = new EBS();
                    //判断是否有触发信号
                    int triggerIndex = 0;
                    EdgeType edgeType1 = ebs.IsTrigger(ref triggerData1, (float)triggerLevel,out triggerIndex);
                    //找到信号1的下降沿信号，所有数据缓存完毕
                    if (edgeType1 == EdgeType.Fall)
                    {
                        ADCStop();

                        //将最后一包数据加入数组中
                        bufLen = packetReceiveSize;
                        tmpBuf = new byte[packetReceiveSize];
                        daqBuffer.Get(ref bufLen, ref tmpBuf);
                        if (bufLen == packetReceiveSize)
                        {
                            data = new DAQData(tmpBuf, bufLen, (float)adc.Amplitude);
                            tmpEbs = new EBSData();
                            tmpEbs.AddLast(data);
                            triggerData1 = tmpEbs.GetTriggerData1();
                            triggerData2 = tmpEbs.GetTriggerData2();
                            signalData1 = tmpEbs.GetSignalData1();
                            signalData2 = tmpEbs.GetSignalData2();

                            TriggerWave1.AddRange(triggerData1);
                            TriggerWave2.AddRange(triggerData2);
                            SignalWave1.AddRange(signalData1);
                            SignalWave2.AddRange(signalData2);
                        }

                        try
                        {
                            ShowTestResult(ref ebs);
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show($"数据处理出错：\n{ex.Message}");
                        }
                    }

                }
            }
        }

        private void ShowTestResult(ref EBS ebs)
        {
            //计算每一个点的步进，单位ms
            double step = 1000.0 / (adc.SampleRate / 4);

            //寻找信号1上下沿触发的位置
            EdgeIndex signalARaise = ebs.FindRaiseIndex(ref TriggerWave1, 0, triggerLevel);
            if(signalARaise.Start==-1 || signalARaise.End == -1)
            {
                MessageBox.Show("未找到SW2的上升沿信号");
                signalARaise.Start = 0;
                signalARaise.End = 0;
            }
            EdgeIndex signalAFall = ebs.FindFallIndex(ref TriggerWave1, signalARaise.End, triggerLevel);
            //寻找信号2上下沿触发的位置
            EdgeIndex signalBFall = ebs.FindFallIndex(ref TriggerWave2, 0, triggerLevel);
            EdgeIndex signalBRaise = ebs.FindRaiseIndex(ref TriggerWave2, signalBFall.End, triggerLevel);

            //计算波形的脉宽
            List<float> waveARaiseWidths = pulseData.GetPluseWidths(ref SignalWave1, 0, signalARaise.Start);
            List<float> waveAFallWidths = pulseData.GetPluseWidths(ref SignalWave1, signalARaise.End, signalBRaise.Start);

            List<float> waveBRaiseWidths = pulseData.GetPluseWidths(ref SignalWave2, 0, signalARaise.Start);
            List<float> waveBFallWidths = pulseData.GetPluseWidths(ref SignalWave2, signalARaise.End, signalBRaise.Start);
            //寻找初始脉宽
            sensor1Pulse.PulseInit = FindMinPulse(ref waveARaiseWidths);
            sensor2Pulse.PulseInit = FindMinPulse(ref waveBRaiseWidths);
            //寻找全行程脉宽
            sensor1Pulse.PulseWhole = FindMaxPulse(ref waveAFallWidths);
            sensor2Pulse.PulseWhole = FindMaxPulse(ref waveBFallWidths);

            //寻找信号1在触发1上升前后脉宽
            EdgePluse edgeA1Pulse = ebs.FindPluseWidth(ref SignalWave1, signalLevel,
                adc.SampleRate, signalARaise);
            lineA11.X = edgeA1Pulse.StartIndex.Start * step;
            lineA12.X = edgeA1Pulse.StartIndex.End * step;
            lineA21.X = edgeA1Pulse.EndIndex.Start * step;
            lineA22.X = edgeA1Pulse.EndIndex.End * step;
            sensor1Pulse.PulseBeforeRaise = edgeA1Pulse.StartPluse;
            sensor1Pulse.PulseAfterRaise = edgeA1Pulse.EndPluse;
            //寻找信号1在触发2上升前后脉宽
            EdgePluse edgeA2Pulse = ebs.FindPluseWidth(ref SignalWave1, signalLevel,
                adc.SampleRate, signalBRaise);
            lineA31.X = edgeA2Pulse.StartIndex.Start * step;
            lineA32.X = edgeA2Pulse.StartIndex.End * step;
            lineA41.X = edgeA2Pulse.EndIndex.Start * step;
            lineA42.X = edgeA2Pulse.EndIndex.End * step;
            sensor1Pulse.PulseBeforeFall = edgeA2Pulse.StartPluse;
            sensor1Pulse.PulseAfterFall = edgeA2Pulse.EndPluse;

            //寻找信号2在触发1上升前后脉宽
            EdgePluse edgeB1Pulse = ebs.FindPluseWidth(ref SignalWave2, signalLevel,
                adc.SampleRate, signalARaise);
            lineB11.X = edgeB1Pulse.StartIndex.Start * step;
            lineB12.X = edgeB1Pulse.StartIndex.End * step;
            lineB21.X = edgeB1Pulse.EndIndex.Start * step;
            lineB22.X = edgeB1Pulse.EndIndex.End * step;
            sensor2Pulse.PulseBeforeRaise = edgeB1Pulse.StartPluse;
            sensor2Pulse.PulseAfterRaise = edgeB1Pulse.EndPluse;
            //寻找信号2在触发2上升前后脉宽
            EdgePluse edgeB2Pulse = ebs.FindPluseWidth(ref SignalWave2, signalLevel,
                adc.SampleRate, signalBRaise);
            lineB31.X = edgeB2Pulse.StartIndex.Start * step;
            lineB32.X = edgeB2Pulse.StartIndex.End * step;
            lineB41.X = edgeB2Pulse.EndIndex.Start * step;
            lineB42.X = edgeB2Pulse.EndIndex.End * step;
            sensor2Pulse.PulseBeforeFall = edgeB2Pulse.StartPluse;
            sensor2Pulse.PulseAfterFall = edgeB2Pulse.EndPluse;

            //将步进改为us
            step *= 1000;
            //信号1上升，信号2下降的时间差
            DtDist1 = signalBFall.End - signalARaise.Start;
            DtDist1 *= step;

            //信号2上升，信号1下降的时间差
            DtDist2 = signalAFall.End - signalBRaise.Start;
            DtDist2 *= step;
            
            //展示数据
            Invoke((DlgtUpdatePlot)UpdatePlot);

            //展示测试结果
            string tempStr;
            bool isPass = configs.Test(sensor1Pulse, sensor2Pulse, DtDist1, DtDist2,out tempStr);
            TestResultEnum testResult = TestResultEnum.Idle;
            if (isPass)
            {
                testResult = TestResultEnum.Pass;
            }
            else
            {
                testResult = TestResultEnum.Fail;
            }
            SaveTestResult(isPass, tempStr);
            Invoke((DlgtUpdateTestResult)UpdateTestResult, testResult);
        }

        private void SaveTestResult(bool isPass,string result)
        {
            string folder = $"{Directory.GetCurrentDirectory()}\\TestResult";
            if (!Directory.Exists(folder))
            {
                Directory.CreateDirectory(folder);
            }
            string nowTime = DateTime.Now.ToString("yyyyMMddhhmmss");
            string filename = $"{folder}\\{nowTime}-{(isPass?"PASS":"FAIL")}.txt";
            StreamWriter writer = null;
            try
            {
                writer = new StreamWriter(filename, true);

                writer.WriteLine($"信号电平:{configs.SignalLevel} v");
                writer.WriteLine($"触发电平:{configs.TriggerLevel} v");
                writer.Write(result);
                
                //writer.WriteLine("---------------信号1测试结果---------------");
                //writer.WriteLine($"行程#1触发前:{sensor1Pulse.PulseBeforeRaise}, min:{configs.S1.BeforeTri.Min}, max:{configs.S1.BeforeTri.Max}");
                //writer.WriteLine($"行程#1触发后:{sensor1Pulse.PulseAfterRaise}, min:{configs.S1.AfterTri.Min}, max:{configs.S1.AfterTri.Max}");
                //writer.WriteLine($"行程#1断开前:{sensor1Pulse.PulseBeforeFall}, min:{configs.S1.BeforeOpen.Min}, max:{configs.S1.BeforeOpen.Max}");
                //writer.WriteLine($"行程#1断开后:{sensor1Pulse.PulseAfterFall}, min:{configs.S1.AfterOpen.Min}, max:{configs.S1.AfterOpen.Max}");
                //writer.WriteLine($"行程#1初始行程:{sensor1Pulse.PulseInit}, min:{configs.S1.Init.Min}, max:{configs.S1.Init.Max}");
                //writer.WriteLine($"行程#1全行程:{sensor1Pulse.PulseWhole}, min:{configs.S1.Whole.Min}, max:{configs.S1.Whole.Max}");
                //writer.WriteLine("---------------信号2测试结果---------------");
                //writer.WriteLine($"行程#2触发前:{sensor2Pulse.PulseBeforeRaise}, min:{configs.S2.BeforeTri.Min}, max:{configs.S2.BeforeTri.Max}");
                //writer.WriteLine($"行程#2触发后:{sensor2Pulse.PulseAfterRaise}, min:{configs.S2.AfterTri.Min}, max:{configs.S2.AfterTri.Max}");
                //writer.WriteLine($"行程#2断开前:{sensor2Pulse.PulseBeforeFall}, min:{configs.S2.BeforeOpen.Min}, max:{configs.S2.BeforeOpen.Max}");
                //writer.WriteLine($"行程#2断开后:{sensor2Pulse.PulseAfterFall}, min:{configs.S2.AfterOpen.Min}, max:{configs.S2.AfterOpen.Max}");
                //writer.WriteLine($"行程#2初始行程:{sensor2Pulse.PulseInit}, min:{configs.S2.Init.Min}, max:{configs.S2.Init.Max}");
                //writer.WriteLine($"行程#2全行程:{sensor2Pulse.PulseWhole}, min:{configs.S2.Whole.Min}, max:{configs.S2.Whole.Max}");
                //writer.WriteLine("---------------开关触发时间差测试结果---------------");
                //writer.WriteLine($"开关闭合时间差:{DtDist1}, min:{configs.SWClose.Min}, max:{configs.SWClose.Max}");
                //writer.WriteLine($"开关断开时间差:{DtDist2}, min:{configs.SWOpen.Min}, max:{configs.SWOpen.Max}");

                writer.Flush();
            }
            catch (Exception ex)
            {
                MessageBox.Show($"保存测试结果时出错！\n{ex.Message}");
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
            }
        }

        #endregion

        private LinearAxis Xaxis1 = new LinearAxis();
        private LinearAxis Yaxis1 = new LinearAxis();
        private LinearAxis Xaxis2 = new LinearAxis();
        private LinearAxis Yaxis2 = new LinearAxis();
        private void InitOxyPlot()
        {
            //=================== pv1 ===================
            pnlPlotView1.Controls.Add(pv1);
            pv1.Dock = DockStyle.Fill;

            //------------------Xaxis----------------
            Xaxis1.Key = "X";
            Xaxis1.Title = "传感器1 Time (ms)";
            Xaxis1.Position = AxisPosition.Bottom;
            Xaxis1.Minimum = 0;
            Xaxis1.Maximum = 30;
            Xaxis1.MajorGridlineStyle = LineStyle.Solid;
            Xaxis1.MinorGridlineStyle = LineStyle.Dot;
            Xaxis1.MajorGridlineColor = OxyColor.FromAColor(192, OxyColors.White);
            Xaxis1.MinorGridlineColor = OxyColor.FromAColor(128, OxyColors.WhiteSmoke);
            Xaxis1.IntervalLength = 50;
            //------------------Yaxis----------------
            Yaxis1.Key = "Y";
            Yaxis1.Title = "Volt (V)";
            Yaxis1.Position = AxisPosition.Left;
            Yaxis1.Minimum = -10;
            Yaxis1.Maximum = 10;
            //Yaxis1.MajorStep = 5;
            Yaxis1.MajorGridlineStyle = LineStyle.Solid;
            Yaxis1.MinorGridlineStyle = LineStyle.Dot;
            Yaxis1.MajorGridlineColor = OxyColor.FromAColor(192, OxyColors.White);
            Yaxis1.MinorGridlineColor = OxyColor.FromAColor(128, OxyColors.WhiteSmoke);

            PlotModel myModel1 = new PlotModel { Title = "" };
            myModel1.Axes.Add(Xaxis1);
            myModel1.Axes.Add(Yaxis1);

            lineASignal.Title = "(ai1)signal1";
            lineASignal.Background = OxyColor.FromAColor(255, OxyColors.Black);
            lineASignal.Color = OxyColor.FromAColor(200, OxyColors.Yellow);
            myModel1.Series.Add(lineASignal);

            lineATri1.Title = "(ai0)SW2";
            lineATri1.Background = OxyColor.FromAColor(255, OxyColors.Black);
            lineATri1.Color = OxyColor.FromAColor(255, OxyColors.Blue);
            myModel1.Series.Add(lineATri1);

            lineATri2.Title = "(ai2)SW1";
            lineATri2.Background = OxyColor.FromAColor(255,OxyColors.Black);
            lineATri2.Color = OxyColor.FromAColor(255, OxyColors.Green);
            myModel1.Series.Add(lineATri2);

            //用于定位测试的脉冲
            lineA11.Type = LineAnnotationType.Vertical;
            lineA11.X = 0;
            lineA11.Color = OxyColors.Orange;
            myModel1.Annotations.Add(lineA11);

            lineA12.Type = LineAnnotationType.Vertical;
            lineA12.X = 0;
            lineA12.Color = OxyColors.Orange;
            myModel1.Annotations.Add(lineA12);
            
            lineA21.Type = LineAnnotationType.Vertical;
            lineA21.X = 0;
            lineA21.Color = OxyColors.Red;
            myModel1.Annotations.Add(lineA21);
                        
            lineA22.Type = LineAnnotationType.Vertical;
            lineA22.X = 0;
            lineA22.Color = OxyColors.Red;
            myModel1.Annotations.Add(lineA22);

            lineA31.Type = LineAnnotationType.Vertical;
            lineA31.X = 0;
            lineA31.Color = OxyColors.Orange;
            myModel1.Annotations.Add(lineA31);

            lineA32.Type = LineAnnotationType.Vertical;
            lineA32.X = 0;
            lineA32.Color = OxyColors.Orange;
            myModel1.Annotations.Add(lineA32);

            lineA41.Type = LineAnnotationType.Vertical;
            lineA41.X = 0;
            lineA41.Color = OxyColors.Red;
            myModel1.Annotations.Add(lineA41);

            lineA42.Type = LineAnnotationType.Vertical;
            lineA42.X = 0;
            lineA42.Color = OxyColors.Red;
            myModel1.Annotations.Add(lineA42);
            //---------- add model ----------
            myModel1.LegendBackground = OxyColor.FromArgb(200, 255, 255, 255);
            pv1.Model = myModel1;
            pv1.Show();

            //=================== pv2 ===================
            pnlPlotView2.Controls.Add(pv2);
            pv2.Dock = DockStyle.Fill;

            //------------------Xaxis----------------
            Xaxis2.Key = "X";
            Xaxis2.Title = "传感器2 Time (ms)";
            Xaxis2.Position = AxisPosition.Bottom;
            Xaxis2.Minimum = 0;
            Xaxis2.Maximum = 30;
            Xaxis2.MajorGridlineStyle = LineStyle.Solid;
            Xaxis2.MinorGridlineStyle = LineStyle.Dot;
            Xaxis2.IntervalLength = 50;
            Xaxis2.MajorGridlineColor = OxyColor.FromAColor(192, OxyColors.White);
            Xaxis2.MinorGridlineColor = OxyColor.FromAColor(128, OxyColors.WhiteSmoke);
            //------------------Yaxis----------------
            Yaxis2.Key = "Y";
            Yaxis2.Title = "Volt (V)";
            Yaxis2.Position = AxisPosition.Left;
            Yaxis2.Minimum = -10;
            Yaxis2.Maximum = 10;
            //Yaxis2.MajorStep = 5;
            Yaxis2.MajorGridlineStyle = LineStyle.Solid;
            Yaxis2.MinorGridlineStyle = LineStyle.Dot;
            Yaxis2.MajorGridlineColor = OxyColor.FromAColor(192, OxyColors.White);
            Yaxis2.MinorGridlineColor = OxyColor.FromAColor(128, OxyColors.WhiteSmoke);

            PlotModel myModel2 = new PlotModel { Title = "" };
            myModel2.Axes.Add(Xaxis2);
            myModel2.Axes.Add(Yaxis2);

            //-------- add line --------
            //line3.InterpolationAlgorithm = InterpolationAlgorithms.CanonicalSpline;
            lineBSignal.Title = "(ai3)Signal2";
            lineBSignal.Background = OxyColor.FromAColor(255, OxyColors.Black);
            lineBSignal.Color = OxyColor.FromAColor(200, OxyColors.Yellow);
            myModel2.Series.Add(lineBSignal);

            //line4.InterpolationAlgorithm = InterpolationAlgorithms.CanonicalSpline;
            lineBTri1.Title = "(ai0)SW2";
            lineBTri1.Background = OxyColor.FromAColor(255, OxyColors.Black);
            lineBTri1.Color = OxyColor.FromAColor(255, OxyColors.Blue);
            myModel2.Series.Add(lineBTri1);

            lineBTri2.Title = "(ai2)SW1";
            lineBTri2.Background = OxyColor.FromAColor(255, OxyColors.Black);
            lineBTri2.Color = OxyColor.FromAColor(255, OxyColors.Green);
            myModel2.Series.Add(lineBTri2);

            lineASignal.Decimator = Decimator.Decimate;
            lineBSignal.Decimator = Decimator.Decimate;
            lineATri1.Decimator = Decimator.Decimate;
            lineATri2.Decimator = Decimator.Decimate;
            lineBTri1.Decimator = Decimator.Decimate;
            lineBTri2.Decimator = Decimator.Decimate;

            //------------ 用于定位测试的脉冲 ------------
            lineB11.Type = LineAnnotationType.Vertical;
            lineB11.X = 0;
            lineB11.Color = OxyColors.Orange;
            myModel2.Annotations.Add(lineB11);

            lineB12.Type = LineAnnotationType.Vertical;
            lineB12.X = 0;
            lineB12.Color = OxyColors.Orange;
            myModel2.Annotations.Add(lineB12);

            lineB21.Type = LineAnnotationType.Vertical;
            lineB21.X = 0;
            lineB21.Color = OxyColors.Red;
            myModel2.Annotations.Add(lineB21);

            lineB22.Type = LineAnnotationType.Vertical;
            lineB22.X = 0;
            lineB22.Color = OxyColors.Red;
            myModel2.Annotations.Add(lineB22);

            lineB31.Type = LineAnnotationType.Vertical;
            lineB31.X = 0;
            lineB31.Color = OxyColors.Orange;
            myModel2.Annotations.Add(lineB31);

            lineB32.Type = LineAnnotationType.Vertical;
            lineB32.X = 0;
            lineB32.Color = OxyColors.Orange;
            myModel2.Annotations.Add(lineB32);

            lineB41.Type = LineAnnotationType.Vertical;
            lineB41.X = 0;
            lineB41.Color = OxyColors.Red;
            myModel2.Annotations.Add(lineB41);

            lineB42.Type = LineAnnotationType.Vertical;
            lineB42.X = 0;
            lineB42.Color = OxyColors.Red;
            myModel2.Annotations.Add(lineB42);
            //---------- add model ----------
            myModel2.LegendBackground = OxyColor.FromArgb(200, 255, 255, 255);
            pv2.Model = myModel2;
            pv2.Show();
        }

        private delegate void UpdateAxisDelegate(double min, double max);
        private void UpdateXAxis(double min, double max)
        {
            if (Xaxis1 == null || Xaxis2 == null)
            {
                MessageBox.Show("没有X轴可以调整");
                return;
            }
            //Xaxis1.Position = AxisPosition.Bottom;
            Xaxis1.Minimum = min;
            Xaxis1.Maximum = max;
            Xaxis1.Reset();

            //Xaxis2.Position = AxisPosition.Bottom;
            Xaxis2.Minimum = min;
            Xaxis2.Maximum = max;
            Xaxis2.Reset();
        }

        private void UpdateYAxis(double min, double max)
        {
            if (Yaxis1 == null || Yaxis2 == null)
            {
                MessageBox.Show("没有Y轴可以调整");
                return;
            }
            //Yaxis1.Position = AxisPosition.Left;
            Yaxis1.Minimum = min;
            Yaxis1.Maximum = max;
            Yaxis1.Reset();

            //Yaxis2.Position = AxisPosition.Left;
            Yaxis2.Minimum = min;
            Yaxis2.Maximum = max;
            Yaxis2.Reset();
        }

        private void cmbDev_SelectedIndexChanged(object sender, EventArgs e)
        {
            adc.Device = cmbDev.Text;
        }

        private void UpdateControlState(bool isEnable)
        {
            Invoke((DlgtSetControlState)SetControlState, new object[] { cmbDev, isEnable });
        }

        private delegate void DlgtSetControlState(Control ctrl, bool state);
        private void SetControlState(Control ctrl, bool state)
        {
            ctrl.Enabled = state;
        }

        private void ClearDgv()
        {
            txtS1BeforeTri.Text = "0";
            txtS1AfterTri.Text = "0";
            txtS1BeforeOpen.Text = "0";
            txtS1AfterOpen.Text = "0";
            txtS1Init.Text = "0";
            txtS1Whole.Text = "0";

            txtS2BeforeTri.Text = "0";
            txtS2AfterTri.Text = "0";
            txtS2BeforeOpen.Text = "0";
            txtS2AfterOpen.Text = "0";
            txtS2Init.Text = "0";
            txtS2Whole.Text = "0";

            txtSWClose.Text = "0";
            txtSWOpen.Text = "0";
        }

        private void UpdateDgv()
        {
            txtS1BeforeTri.Text = sensor1Pulse.PulseBeforeRaise.ToString();
            txtS1AfterTri.Text = sensor1Pulse.PulseAfterRaise.ToString();
            txtS1BeforeOpen.Text = sensor1Pulse.PulseBeforeFall.ToString();
            txtS1AfterOpen.Text = sensor1Pulse.PulseAfterFall.ToString();
            txtS1Init.Text = sensor1Pulse.PulseInit.ToString();
            txtS1Whole.Text = sensor1Pulse.PulseWhole.ToString();

            txtS2BeforeTri.Text = sensor2Pulse.PulseBeforeRaise.ToString();
            txtS2AfterTri.Text = sensor2Pulse.PulseAfterRaise.ToString();
            txtS2BeforeOpen.Text = sensor2Pulse.PulseBeforeFall.ToString();
            txtS2AfterOpen.Text = sensor2Pulse.PulseAfterFall.ToString();
            txtS2Init.Text = sensor2Pulse.PulseInit.ToString();
            txtS2Whole.Text = sensor2Pulse.PulseWhole.ToString();

            txtSWClose.Text = DtDist1.ToString();
            txtSWOpen.Text = DtDist2.ToString();
        }

    }
}
