﻿using Finetooling.DAL;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DataAcquisitionCardDemo.ADC
{
    class EBSData
    {
        public LinkedList<float[]> TriggerData1;
        public LinkedList<float[]> SignalData1;
        public LinkedList<float[]> TriggerData2;
        public LinkedList<float[]> SignalData2;

        public int Count { get; set; }

        public int Length { get; set; }

        public EBSData()
        {
            TriggerData1 = new LinkedList<float[]>();
            SignalData1 = new LinkedList<float[]>();
            TriggerData2 = new LinkedList<float[]>();
            SignalData2 = new LinkedList<float[]>();
            Count = 0;
        }

        public void Clear()
        {
            ClearLinked(ref TriggerData1);
            ClearLinked(ref SignalData1);
            ClearLinked(ref TriggerData2);
            ClearLinked(ref SignalData2);
            Count = 0;
        }

        private void ClearLinked(ref LinkedList<float[]> linked)
        {
            if (linked == null)
            {
                linked = new LinkedList<float[]>();
            }
            else
            {
                linked.Clear();
            }
        }

        public bool AddLast(DAQData data)
        {
            if (data != null && (data.Len % 8) == 0)
            {
                float[] values = data.ToFloats();
                int traceLen = values.Length / 4;

                float[] trigger1 = new float[traceLen];
                float[] signal1 = new float[traceLen];
                float[] trigger2 = new float[traceLen];
                float[] signal2 = new float[traceLen];

                for (int i = 0; i < traceLen; i++)
                {
                    trigger1[i] = values[i * 4];
                    signal1[i] = values[i * 4 + 1];
                    trigger2[i] = values[i * 4 + 2];
                    signal2[i] = values[i * 4 + 3];
                }

                TriggerData1.AddLast(trigger1);
                SignalData1.AddLast(signal1);
                TriggerData2.AddLast(trigger2);
                SignalData2.AddLast(signal2);
                Length += traceLen;

                Count++;

                return true;
            }

            return false;
        }

        public void RemoveFirst()
        {
            if (Count <= 0)
            {
                return;
            }

            Length -= TriggerData1.First.Value.Length;

            TriggerData1.RemoveFirst();
            SignalData1.RemoveFirst();
            TriggerData2.RemoveFirst();
            SignalData2.RemoveFirst();

            Count--;
        }

        public float[] GetTriggerData1()
        {
            return GetDataElements(ref TriggerData1);
        }

        public float[] GetTriggerData2()
        {
            return GetDataElements(ref TriggerData2);
        }

        public float[] GetSignalData1()
        {
            return GetDataElements(ref SignalData1);
        }

        public float[] GetSignalData2()
        {
            return GetDataElements(ref SignalData2);
        }

        private float[] GetDataElements(ref LinkedList<float[]> linkedList)
        {
            int totalLen = 0;
            foreach(float[] data in linkedList)
            {
                totalLen += data.Length;
            }

            float[] datas = new float[totalLen];
            int index = 0;
            foreach (float[] data in linkedList)
            {
                Array.Copy(data,0,datas,index,data.Length);
                index += data.Length;
            }

            return datas;
        }
    }
}
