﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DataAcquisitionCardDemo.ADC
{
    /// <summary>
    /// 用以处理脉宽信号，计算脉宽值
    /// </summary>
    public class PulseData
    {
        //脉宽信号数据大于该值为脉宽
        private double pulseLevel=1.0;

        //一个脉宽信号占的时间长度
        private double step=5;

        public PulseData()
        {
            
        }

        public void SetParam(double level,double sampleRate)
        {
            this.pulseLevel = level;
            step = 1000000.0 / (sampleRate / 4);
        }

        public List<float> GetPluseWidths(ref List<float> pluseData,int startIndex, int endIndex)
        {
            List<float> pluseWidths = new List<float>();
            if (pluseData == null || startIndex < 0 || endIndex < 0 || endIndex >= pluseData.Count)
            {
                return pluseWidths;
            }

            //找到最后一个脉宽的前一个点的位置
            if (pluseData[endIndex] >= pulseLevel)
            {
                //寻找最后一个低电平的点
                while (endIndex > startIndex)
                {
                    endIndex--;
                    if (pluseData[endIndex] < pulseLevel)
                    {
                        break;
                    }
                }
            }

            int index = startIndex + 1;
            while(index < endIndex)
            {
                //发现沿跳变，记录i点
                if(pluseData[index] >= pulseLevel && pluseData[index - 1] < pulseLevel)
                {
                    int count = 0;
                    while(index < endIndex)
                    {
                        if(pluseData[index] >= pulseLevel)
                        {
                            count++;
                        }
                        else    //脉宽结束
                        {
                            //保存脉宽
                            pluseWidths.Add((float)(count * step));
                            break;
                        }
                        index++;
                    }
                }
                else
                {
                    index++;
                }
            }

            return pluseWidths;
        }
    }
}
