﻿using DataAcquisitionCardDemo.ADC;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ZhDZ.IniUtils;

namespace DataAcquisitionCardDemo.Config
{
    public class ConfigParams: INotifyPropertyChanged
    {
        private float _SignalLevel = 1;
        private float _TriggerLevel = 4;

        private SignalParam _S1 = new SignalParam();
        private SignalParam _S2 = new SignalParam();

        private ParamBound _SWClose = new ParamBound();
        private ParamBound _SWOpen = new ParamBound();

        public float SignalLevel
        {
            get
            {
                return _SignalLevel;
            }

            set
            {
                _SignalLevel = value;
                NotifyPropertyChanged("SignalLevel");
            }
        }

        public float TriggerLevel
        {
            get
            {
                return _TriggerLevel;
            }

            set
            {
                _TriggerLevel = value;
                NotifyPropertyChanged("TriggerLevel");
            }
        }

        public SignalParam S1
        {
            get
            {
                return _S1;
            }

            set
            {
                _S1 = value;
                NotifyPropertyChanged("S1");
            }
        }

        public SignalParam S2
        {
            get
            {
                return _S2;
            }

            set
            {
                _S2 = value;
                NotifyPropertyChanged("S2");
            }
        }

        public ParamBound SWClose
        {
            get
            {
                return _SWClose;
            }

            set
            {
                _SWClose = value;
                NotifyPropertyChanged("SWClose");
            }
        }

        public ParamBound SWOpen
        {
            get
            {
                return _SWOpen;
            }

            set
            {
                _SWOpen = value;
                NotifyPropertyChanged("SWOpen");
            }
        }

        public ConfigParams()
        {
            GetFromIni();
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public bool Test(SensorPulse sensor1, SensorPulse sensor2,double dt1,double dt2,out string result)
        {
            bool tempBool = false;
            bool isPass = true;
            string tempStr = "";

            StringBuilder sb = new StringBuilder();
            tempBool = TestSignalBounds(ref _S1, ref sensor1,out tempStr);
            sb.Append($"-----------信号1测试结果:{(tempBool ? "Pass" : "Fail")}-----------\r\n");
            sb.Append($"{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }

            tempBool = TestSignalBounds(ref _S2, ref sensor2, out tempStr);
            sb.Append($"-----------信号2测试结果:{(tempBool ? "Pass" : "Fail")}-----------\r\n");
            sb.Append($"{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }

            sb.Append("-------开关触发时间差测试结果-------\r\n");

            tempBool = SWClose.IsInBounds(dt1, out tempStr);
            sb.Append($"{tempBool}-开关闭合时间差:{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }
            tempBool = SWOpen.IsInBounds(dt2, out tempStr);
            sb.Append($"{tempBool}-开关断开时间差:{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }

            result = sb.ToString();
            return isPass;
        }

        private bool TestSignalBounds(ref SignalParam signal,ref SensorPulse sensor,out string result)
        {
            bool isPass = true;
            bool tempBool = false;
            string tempStr = "";
            string prefix = "行程#";
            if (signal == _S1)
            {
                prefix = "行程#1";
            }
            else if (signal == _S2)
            {
                prefix = "行程#2";
            }
            StringBuilder sb = new StringBuilder();

            tempBool = signal.BeforeTri.IsInBounds(sensor.PulseBeforeRaise, out tempStr);
            sb.Append($"{tempBool}-{prefix}触发前:{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }

            tempBool = signal.AfterTri.IsInBounds(sensor.PulseAfterRaise, out tempStr);
            sb.Append($"{tempBool}-{prefix}触发后:{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }

            tempBool = signal.BeforeOpen.IsInBounds(sensor.PulseBeforeFall, out tempStr);
            sb.Append($"{tempBool}-{prefix}断开前:{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }

            tempBool = signal.AfterOpen.IsInBounds(sensor.PulseAfterFall, out tempStr);
            sb.Append($"{tempBool}-{prefix}断开后:{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }

            tempBool = signal.Init.IsInBounds(sensor.PulseInit, out tempStr);
            sb.Append($"{tempBool}-{prefix}初始行程:{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }

            tempBool = signal.Whole.IsInBounds(sensor.PulseWhole, out tempStr);
            sb.Append($"{tempBool}-{prefix}全行程:{tempStr}\r\n");
            if (!tempBool)
            {
                isPass = false;
            }

            result = sb.ToString();
            return isPass;
        }

        public void GetFromIni()
        {
            float tempFloat = 0;
            if (float.TryParse(IniHelp.GetValue("Level","SignalLevel", "1"), out tempFloat))
            {
                SignalLevel = tempFloat;
            }
            if (float.TryParse(IniHelp.GetValue("Level", "TriggerLevel", "4"), out tempFloat))
            {
                TriggerLevel = tempFloat;
            }

            GetIni(ref _S1,"S1");
            GetIni(ref _S2, "S2");

            double tempDouble = 0;
            if (double.TryParse(IniHelp.GetValue("SW", "SWCloseMin", "1000"), out tempDouble))
            {
                _SWClose.Min = tempDouble;
            }
            if (double.TryParse(IniHelp.GetValue("SW", "SWCloseMax", "4000"), out tempDouble))
            {
                _SWClose.Max = tempDouble;
            }
            if (double.TryParse(IniHelp.GetValue("SW", "SWOpenMin", "1000"), out tempDouble))
            {
                _SWOpen.Min = tempDouble;
            }
            if (double.TryParse(IniHelp.GetValue("SW", "SWOpenMax", "4000"), out tempDouble))
            {
                _SWOpen.Max = tempDouble;
            }
        }

        private void GetIni(ref SignalParam signal,string section)
        {
            double temp = 0;
            if (double.TryParse(IniHelp.GetValue(section, "BeforeTriMin", "650"), out temp))
            {
                signal.BeforeTri.Min = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "BeforeTriMax", "1300"), out temp))
            {
                signal.BeforeTri.Max = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "AfterTriMin", "650"), out temp))
            {
                signal.AfterTri.Min = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "AfterTriMax", "1300"), out temp))
            {
                signal.AfterTri.Max = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "BeforeOpenMin", "650"), out temp))
            {
                signal.BeforeOpen.Min = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "BeforeOpenMax", "1300"), out temp))
            {
                signal.BeforeOpen.Max = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "AfterOpenMin", "650"), out temp))
            {
                signal.AfterOpen.Min = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "AfterOpenMax", "1300"), out temp))
            {
                signal.AfterOpen.Max = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "InitMin", "400"), out temp))
            {
                signal.Init.Min = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "InitMax", "800"), out temp))
            {
                signal.Init.Max = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "WholeMin", "800"), out temp))
            {
                signal.Whole.Min = temp;
            }
            if (double.TryParse(IniHelp.GetValue(section, "WholeMax", "4000"), out temp))
            {
                signal.Whole.Max = temp;
            }
        }

        public void SetToIni()
        {
            IniHelp.SetValue("Level", "SignalLevel", SignalLevel.ToString());
            IniHelp.SetValue("Level", "TriggerLevel", TriggerLevel.ToString());

            SetIni(ref _S1, "S1");
            SetIni(ref _S2, "S2");

            IniHelp.SetValue("SW", "SWCloseMin", _SWClose.Min.ToString());
            IniHelp.SetValue("SW", "SWCloseMax", _SWClose.Max.ToString());
            IniHelp.SetValue("SW", "SWOpenMin", _SWOpen.Min.ToString());
            IniHelp.SetValue("SW", "SWOpenMax", _SWOpen.Max.ToString());
        }

        private void SetIni(ref SignalParam signal, string section)
        {
            IniHelp.SetValue(section, "BeforeTriMin", signal.BeforeTri.Min.ToString());
            IniHelp.SetValue(section, "BeforeTriMax", signal.BeforeTri.Max.ToString());
            IniHelp.SetValue(section, "AfterTriMin", signal.AfterTri.Min.ToString());
            IniHelp.SetValue(section, "AfterTriMax", signal.AfterTri.Max.ToString());

            IniHelp.SetValue(section, "BeforeOpenMin", signal.BeforeOpen.Min.ToString());
            IniHelp.SetValue(section, "BeforeOpenMax", signal.BeforeOpen.Max.ToString());
            IniHelp.SetValue(section, "AfterOpenMin", signal.AfterOpen.Min.ToString());
            IniHelp.SetValue(section, "AfterOpenMax", signal.AfterOpen.Max.ToString());

            IniHelp.SetValue(section, "InitMin", signal.Init.Min.ToString());
            IniHelp.SetValue(section, "InitMax", signal.Init.Max.ToString());
            IniHelp.SetValue(section, "WholeMin", signal.Whole.Min.ToString());
            IniHelp.SetValue(section, "WholeMax", signal.Whole.Max.ToString());
        }
    }

    public class SignalParam
    {
        public ParamBound BeforeTri = new ParamBound();
        public ParamBound AfterTri = new ParamBound();
        public ParamBound BeforeOpen = new ParamBound();
        public ParamBound AfterOpen = new ParamBound();
        public ParamBound Init = new ParamBound();
        public ParamBound Whole = new ParamBound();

        public SignalParam()
        {

        }
    }

    public class ParamBound : INotifyPropertyChanged
    {
        private double _Min = 0;
        private double _Max = 0;

        public ParamBound()
        {

        }

        public ParamBound(double min,double max)
        {
            this._Min = min;
            this._Max = max;
        }

        public double Min
        {
            get
            {
                return _Min;
            }

            set
            {
                _Min = value;
                NotifyPropertyChanged("Min");
            }
        }
        public double Max
        {
            get
            {
                return _Max;
            }

            set
            {
                _Max = value;
                NotifyPropertyChanged("Max");
            }
        }

        public bool IsInBounds(double value,out string result)
        {
            bool isPass = true;
            if (value >= Min && value <= Max)
            {
                isPass = true;
            }
            else
            {
                isPass = false;
            }

            result = $"{value},(min:{Min}, max:{Max})";
            return isPass;
        }

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
