﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using CyUSB;
using System.Windows.Forms.DataVisualization.Charting;
using DataAcquisitionCardDemo.DAL;
using DataAcquisitionCardDemo.Model;
using System.Threading;
using DataAcquisitionCardDemo.View;
using DataAcquisitionCardDemo.Service;

namespace DataAcquisitionCardDemo
{
    public partial class MainForm : Form
    {
        public static USBDeviceList devices = null;
        public static CyUSBDevice curDevice = null;
        public static CyBulkEndPoint outEndPoint = null;
        public static CyBulkEndPoint intEndPoint = null;

        private static byte DIODeriction = 0xFF; // 1为输出，0为输入
        private static byte DIOOutputState = 0;

        private List<Button> listDirection = new List<Button>(8);
        private List<Button> listDIOOutput = new List<Button>(8);
        private List<Label> listDIOInput = new List<Label>(8);

        DACConfig daqConfig = new DACConfig();

        private int points = 1600;

        private double[] curveData;
        short[] frameData;
        private int ch = 0;

        bool IsContinueRead = false;

        bool bFreqUpdate = false;
        bool bVppUpdate = false;
        bool bTHDUpdate = false;

        Thread DIOThread;
        Thread ADCThread;

        System.Timers.Timer t1 = new System.Timers.Timer(500);

        public MainForm()
        {
            InitializeComponent();
            tabControl1.SelectedIndex = 1;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            listDirection.Add(btnDIODirection1);
            listDirection.Add(btnDIODirection2);
            listDirection.Add(btnDIODirection3);
            listDirection.Add(btnDIODirection4);
            listDirection.Add(btnDIODirection5);
            listDirection.Add(btnDIODirection6);
            listDirection.Add(btnDIODirection7);
            listDirection.Add(btnDIODirection8);

            listDIOOutput.Add(btnDIOOutput1);
            listDIOOutput.Add(btnDIOOutput2);
            listDIOOutput.Add(btnDIOOutput3);
            listDIOOutput.Add(btnDIOOutput4);
            listDIOOutput.Add(btnDIOOutput5);
            listDIOOutput.Add(btnDIOOutput6);
            listDIOOutput.Add(btnDIOOutput7);
            listDIOOutput.Add(btnDIOOutput8);

            listDIOInput.Add(ledDIOInState1);
            listDIOInput.Add(ledDIOInState2);
            listDIOInput.Add(ledDIOInState3);
            listDIOInput.Add(ledDIOInState4);
            listDIOInput.Add(ledDIOInState5);
            listDIOInput.Add(ledDIOInState6);
            listDIOInput.Add(ledDIOInState7);
            listDIOInput.Add(ledDIOInState8);

            Initialize();

            curveData = new double[points];
            frameData = new short[points];

            //DIOThread = new Thread(GetDIOInput);
            //DIOThread.IsBackground = true;
            //DIOThread.Start();

            //ADCThread = new Thread(GetDataBlock);
            //ADCThread.IsBackground = true;
            //ADCThread.Start();
        }

        private void GetDIOInput()
        {
            BeginInvoke((DlgtGetDIOInputState)GetDIOInputState);
        }

        private void GetDataBlock()
        {
            while (IsContinueRead)
            {
                int bytes = 4096;
                int state = 0;
                state = DAQDAL.SetADCStart(curDevice, Startup.Start);
                //Thread.Sleep(100);
                for (int i = 0; i < 4096 / 512; i++)
                {
                    double[] fram = new double[200];
                    DAQDAL.ReadFrameData(intEndPoint, ref fram);
                    Buffer.BlockCopy(fram, 0, curveData, 200 * i * sizeof(double), 200 * sizeof(double));
                }

                state = DAQDAL.SetADCStart(curDevice, Startup.Stop);
                //DAQDAL.ReadBlockData(intEndPoint, ref curveData);

                //CatchData();
                //for (int k = 0; k < points; k++)
                //{
                //    curveData[k] = frameData[k] / 32768.0 * 10;
                //}

                CalculateAudioParameter();
                BeginInvoke((DlgtUpdateChartADC)UpdateChartADC);
                BeginInvoke((DlgtUpdateAudioParameter)UpdateAudioParameter);
                Thread.Sleep(100);
            }
        }

        double frequency = 0;
        double vpp = 0;
        double thd = 0;

        private void CalculateAudioParameter()
        {
            double[] wave = new double[curveData.Length];
            Buffer.BlockCopy(curveData, 0, wave, 0, curveData.Length * sizeof(double));
            if (bFreqUpdate)
            {
                frequency = AudioProcess.E28CalculateFrequency(wave, 10000, points);
            }
            if (bVppUpdate)
            {
                vpp = AudioProcess.E28CalculateVpp(wave, points);
            }
            if (bTHDUpdate)
            {
                //thd = AudioProcess.ArrayComputeTHD(wave, 10000, 5, points);
                thd = THD.Calculate(wave, -1, 10000, 4);
                //thd = THDN.Calculate(wave, 50, 10000);
            }
        }
        private delegate void DlgtUpdateAudioParameter();
        private void UpdateAudioParameter()
        {
            txtFrequency.Text = bFreqUpdate ? frequency.ToString("F2") : "";
            txtVpp.Text = bVppUpdate ? vpp.ToString("F3") : "";
            txtThd.Text = bTHDUpdate ? thd.ToString("F5") : "";
        }

        private void Initialize()
        {
            InitChartADC();
            ADCForm adc = new ADCForm();
            adc.FormBorderStyle = FormBorderStyle.None;
            adc.TopLevel = false;
            adc.Visible = true;
            adc.Dock = DockStyle.Fill;
            tabADC.Controls.Add(adc);
            adc.Show();
            DACForm dac = new DACForm();
            dac.FormBorderStyle = FormBorderStyle.None;
            dac.TopLevel = false;
            dac.Visible = true;
            dac.Dock = DockStyle.Fill;
            tabDAC2.Controls.Add(dac);
            dac.Show();
            //DIOForm dio = new DIOForm();
            //dio.FormBorderStyle = FormBorderStyle.None;
            //dio.TopLevel = false;
            //dio.Visible = true;
            //dio.Dock = DockStyle.Fill;
            //tabDIO.Controls.Add(dio);
            //dio.Show();


            UpdateLsbDAQDevices();
            for (int i = 0; i < 8; i++)
            {
                UpdateDIOState(i + 1);
            }
            devices = new USBDeviceList(CyConst.DEVICES_CYUSB);
            devices.DeviceAttached += new EventHandler(usbDevices_DeviceAttached);
            devices.DeviceRemoved += new EventHandler(usbDevices_DeviceRemoved);

            cbbChannel.SelectedIndex = 0;
            cbbSignalType.SelectedIndex = 0;
            cbbSignalMode.SelectedIndex = 1;

            t1.Elapsed += new System.Timers.ElapsedEventHandler(TimeoutEvent);//到达时间的时候执行事件；

            t1.AutoReset = true;//设置是执行一次（false）还是一直执行(true)；

            t1.Enabled = false;//是否执行System.Timers.Timer.Elapsed事件；
        }

        public void TimeoutEvent(object source, System.Timers.ElapsedEventArgs e)
        {
            if (intEndPoint != null)
            {
                GetDIOInput();
            }
        }

        private void CatchData()
        {
            int length = 9;

            //byte[] cmdbuf = { 0xAA, 0x07, 0x3, 0x08, 0x01, 0x00, 0x00, 0x00, 0x00 };

            //outEndPoint.TimeOut = 100;
            //bool bSend = outEndPoint.XferData(ref cmdbuf, ref length, true);
            //if (bSend)
            //{
            int bytes = 4096;

            byte[] readbuf = new byte[4096];

            intEndPoint.TimeOut = 1000;

            bool bRecv = intEndPoint.XferData(ref readbuf, ref bytes);

            //txtLength.Text = bytes.ToString();

            if (bRecv == true)
            {
                //Int16[] data = new Int16[2048];

                Buffer.BlockCopy(readbuf, 0, frameData, 0, bytes);

                //StringBuilder sb = new StringBuilder();

                //for (int i = 0; i < 2048; i++)
                //{
                //    sb.Append(string.Format("{0,2} ", frameData[i] / 32768.0 * 10));
                //}
            }
            //txtShow.Text = sb.ToString();
            //}
            //else
            //{

            //}
        }

        /* Summary
            Event handler for new device attach
        */
        void usbDevices_DeviceAttached(object sender, EventArgs e)
        {
            USBEventArgs usbEvent = e as USBEventArgs;

            RefreshDeviceList();
        }

        /* Summary
            Event handler for device removal
        */
        void usbDevices_DeviceRemoved(object sender, EventArgs e)
        {
            USBEventArgs usbEvent = e as USBEventArgs;
            //curEndpt = null; // reinitialize
            RefreshDeviceList();
        }

        private void RefreshDeviceList()
        {
            lsbDAQDevices.Items.Clear();
            //DescText.Text = "";

            devices = new USBDeviceList(CyConst.DEVICES_CYUSB);
            curDevice = null;
            outEndPoint = null;
            intEndPoint = null;

            foreach (USBDevice dev in devices)
                lsbDAQDevices.Items.Add(dev.Name);
        }

        private void UpdateLsbDAQDevices()
        {
            lsbDAQDevices.Items.Clear();
            devices = new USBDeviceList(CyConst.DEVICES_CYUSB);

            foreach (USBDevice item in devices)
            {
                lsbDAQDevices.Items.Add(item.Name);
            }
            if (lsbDAQDevices.Items.Count > 0)
            {
                lsbDAQDevices.SelectedIndex = 0;
                curDevice = devices[lsbDAQDevices.SelectedIndex] as CyUSBDevice;
                outEndPoint = curDevice.BulkOutEndPt;
                intEndPoint = curDevice.BulkInEndPt;
                outEndPoint.TimeOut = 1000;
                intEndPoint.TimeOut = 1000;
            }
        }

        private void lsbDAQDevices_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lsbDAQDevices.SelectedIndex != -1)
            {
                curDevice = devices[lsbDAQDevices.SelectedIndex] as CyUSBDevice;
                outEndPoint = curDevice.BulkOutEndPt;
                intEndPoint = curDevice.BulkInEndPt;
            }
        }

        private void InitChartADC()
        {
            chartADC.ChartAreas[0].BackColor = Color.Black;
            chartADC.ChartAreas[0].Axes[0].MajorGrid.Enabled = true; //X轴上网格
            chartADC.ChartAreas[0].Axes[1].MajorGrid.Enabled = true; //y轴上网格
            chartADC.ChartAreas[0].Axes[0].MajorGrid.LineDashStyle = ChartDashStyle.Dash; //网格类型 短横线
            chartADC.ChartAreas[0].Axes[0].MajorGrid.LineColor = Color.Gray;
            chartADC.ChartAreas[0].Axes[0].MajorTickMark.Enabled = false; // x轴上突出的小点
            //chartADC.ChartAreas[0].AxisX.Interval = 10;
            chartADC.ChartAreas[0].Axes[1].MajorTickMark.Enabled = false; //
            chartADC.ChartAreas[0].Axes[1].IsInterlaced = true; //显示交错带
            //chartADC.ChartAreas[0].Axes[0].LabelStyle.Format = "#年"; //设置X轴显示样式
            chartADC.ChartAreas[0].Axes[1].MajorGrid.LineDashStyle = ChartDashStyle.Dash; //网格类型 短横线
            chartADC.ChartAreas[0].Axes[1].MajorGrid.LineColor = Color.Gray;
            chartADC.ChartAreas[0].Axes[1].MajorGrid.LineWidth = 1;
            chartADC.ChartAreas[0].AxisY.Interval = 4;

            Series series = chartADC.Series[0];

            series.ChartType = SeriesChartType.Spline;
            series.BorderWidth = 1;
            series.Color = System.Drawing.Color.Yellow;


            //series.LegendText = "演示曲线";
            // 准备数据
            float[] values = { 95, 30, 20, 23, 60, 87, 42, 77, 92, 51, 29 };

            // 在chart中显示数据
            int x = 0;
            foreach (float v in values)
            {
                series.Points.AddXY(x, v);
                x++;
            }
            //series.Points.Add(values);

            // 设置显示范围
            ChartArea chartArea = chartADC.ChartAreas[0];
            chartArea.AxisX.Minimum = 0;
            chartArea.AxisX.Maximum = points;
            chartArea.AxisY.Minimum = -10d;
            chartArea.AxisY.Maximum = 10d;
        }

        private delegate void DlgtUpdateChartADC();
        private void UpdateChartADC()
        {
            Series series = chartADC.Series[0];
            series.Points.Clear();
            for (int i = 0; i < points; i++)
            {
                series.Points.Add(curveData[i], i);
            }

        }


        #region DIO

        private void btnDIOOutput1_Click(object sender, EventArgs e)
        {
            DIOOutput(1);
        }

        private void btnDIOOutput2_Click(object sender, EventArgs e)
        {
            DIOOutput(2);
        }

        private void btnDIOOutput3_Click(object sender, EventArgs e)
        {
            DIOOutput(3);
        }

        private void btnDIOOutput4_Click(object sender, EventArgs e)
        {
            DIOOutput(4);
        }

        private void btnDIOOutput5_Click(object sender, EventArgs e)
        {
            DIOOutput(5);
        }

        private void btnDIOOutput6_Click(object sender, EventArgs e)
        {
            DIOOutput(6);
        }

        private void btnDIOOutput7_Click(object sender, EventArgs e)
        {
            DIOOutput(7);
        }

        private void btnDIOOutput8_Click(object sender, EventArgs e)
        {
            DIOOutput(8);
        }

        private int DIOOutput(byte pos)
        {
            if (curDevice == null)
            {
                return DAQInfo.ERROR_OFFLINE;
            }
            byte val = Convert.ToByte(0x01 << (pos - 1));
            if ((DIOOutputState & val) == val)
            {
                DIOOutputState = Convert.ToByte(DIOOutputState & (~val));
                listDIOOutput[pos - 1].Text = "OFF";
                listDIOOutput[pos - 1].BackColor = Color.LightGray;
            }
            else
            {
                DIOOutputState = Convert.ToByte(DIOOutputState | val);
                listDIOOutput[pos - 1].Text = "ON";
                listDIOOutput[pos - 1].BackColor = Color.LightGreen;
            }
            int state = 0;
            state = DAQDAL.SetDIOState(curDevice, 0, DIOOutputState);
            return state;
        }

        private delegate void DlgtGetDIOInputState();
        private void GetDIOInputState()
        {
            if (curDevice == null)
            {
                return;
            }

            //DAQDAL.SetDIODirection(curDevice, 0, ~DIOOutputState);
            int data = 0;
            DAQDAL.GetDIOState(curDevice, 0, ref data);
            byte state = Convert.ToByte(data);
            int curpos = 0x01;
            for (int i = 0; i < 8; i++)
            {
                curpos = 0x01 << (i);
                if ((DIODeriction & curpos) == 1)
                {
                    if ((curpos & state) == 0)
                    {
                        listDIOInput[i].BackColor = Color.Red;
                    }
                    else
                    {
                        listDIOInput[i].BackColor = Color.Green;
                    }
                }
            }
        }

        private void UpdateDIOState(int pos)
        {
            byte bit = Convert.ToByte(0x01 << (pos - 1));
            if ((DIODeriction & bit) == bit)
            {
                DIODeriction = Convert.ToByte(DIODeriction & (~bit));
                listDirection[pos - 1].Text = "out";
                listDIOOutput[pos - 1].Enabled = true;
                listDIOInput[pos - 1].Enabled = false;
                listDIOInput[pos - 1].BackColor = Color.Black;
            }
            else
            {
                DIODeriction = Convert.ToByte(DIODeriction | bit);
                listDirection[pos - 1].Text = "in";
                listDIOOutput[pos - 1].Enabled = false;
                listDIOInput[pos - 1].Enabled = true;
                listDIOInput[pos - 1].BackColor = Color.LightGray;
            }
            int state = 0;
            state = DAQDAL.SetDIODirection(curDevice, 0, DIODeriction);
        }

        //private delegate void DlgtUpdateDIOInputState(Label l, bool bState);
        //private void UpdateDIOInputState(Label l, bool bState)
        //{
        //    l.BackColor = bState ? Color.LightGreen : Color.LightGray;
        //}

        private void btnDIODirection1_Click(object sender, EventArgs e)
        {
            UpdateDIOState(1);
        }

        private void btnDIODirection2_Click(object sender, EventArgs e)
        {
            UpdateDIOState(2);
        }

        private void btnDIODirection3_Click(object sender, EventArgs e)
        {
            UpdateDIOState(3);
        }

        private void btnDIODirection4_Click(object sender, EventArgs e)
        {
            UpdateDIOState(4);
        }

        private void btnDIODirection5_Click(object sender, EventArgs e)
        {
            UpdateDIOState(5);
        }

        private void btnDIODirection6_Click(object sender, EventArgs e)
        {
            UpdateDIOState(6);
        }

        private void btnDIODirection7_Click(object sender, EventArgs e)
        {
            UpdateDIOState(7);
        }

        private void btnDIODirection8_Click(object sender, EventArgs e)
        {
            UpdateDIOState(8);
        }

        #endregion

        private void tsbADCStart_Click(object sender, EventArgs e)
        {
            int state = 0;
            state = DAQDAL.SetADCChannelMode(curDevice, 0, ADCChannelMode.Single);
            state = DAQDAL.SetADCSampleMode(curDevice, ADCSampleMode.Continue);
            state = DAQDAL.SetADCSampleChannel(curDevice, 1);
            state = DAQDAL.SetADCSignalType(curDevice, ADCSignelType.Volt);
            //state = DAQDAL.SetADCStart(curDevice, Startup.Start);
            //state = DAQDAL.SetADCRead(curDevice, 1 );
            IsContinueRead = true;
            ADCThread = new Thread(GetDataBlock);
            ADCThread.IsBackground = true;
            ADCThread.Start();
            tsbADCStart.Enabled = false;
            tsbADCStop.Enabled = true;

            cbbChannel.Enabled = false;
            cbbSignalMode.Enabled = false;
            cbbSignalType.Enabled = false;

        }

        private void tsbADCStop_Click(object sender, EventArgs e)
        {
            IsContinueRead = false;
            DAQDAL.SetADCStart(curDevice, Startup.Stop);

            tsbADCStop.Enabled = false;
            tsbADCStart.Enabled = true;

            cbbChannel.Enabled = true;
            cbbSignalMode.Enabled = true;
            cbbSignalType.Enabled = true;
        }

        private void ckbFrequency_CheckedChanged(object sender, EventArgs e)
        {
            bFreqUpdate = ckbFrequency.Checked;
        }

        private void ckbVpp_CheckedChanged(object sender, EventArgs e)
        {
            bVppUpdate = ckbVpp.Checked;
        }

        private void ckbTHD_CheckedChanged(object sender, EventArgs e)
        {
            bTHDUpdate = ckbTHD.Checked;
        }
    }
}
