﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using CyUSB;
using System.Threading;
using DataAcquisitionCardDemo.ADC;
using DataAcquisitionCardDemo.View;
using DataAcquisitionCardDemo.Config;
using Finetooling.DAL.Model;
using Finetooling.DAL;

namespace DataAcquisitionCardDemo
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// 参数的上下限设置信息，用于ADCForm和ConfigForm之间传输参数
        /// </summary>
        public static ConfigParams configs = new ConfigParams();

        public static USBDeviceList devices = null;

        public static Dictionary<string, DAQDevice> daqs = new Dictionary<string, DAQDevice>();

        public static List<string> devList = new List<string>();

        // 声明委托
        public delegate void DeviceUpdate(List<string> devList);
        // 使用委托声明事件
        public event DeviceUpdate DeviceUpdateEvent;

        private ADCForm adcForm = new ADCForm();
        private ConfigForm configForm = new ConfigForm();

        public MainForm()
        {
            InitializeComponent();
            tabForms.SelectedIndex = 0;

        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            Initialize();
        }

        private void Initialize()
        {
            adcForm.FormBorderStyle = FormBorderStyle.None;
            adcForm.TopLevel = false;
            adcForm.Visible = true;
            adcForm.Dock = DockStyle.Fill;
            tabADC.Controls.Add(adcForm);

            configForm.FormBorderStyle = FormBorderStyle.None;
            configForm.TopLevel = false;
            configForm.Visible = false;
            configForm.Dock = DockStyle.Fill;
            tabConfig.Controls.Add(configForm);

            DeviceUpdateEvent += adcForm.UpdateDevice;

            UpdateLsbDAQDevices();

            devices = new USBDeviceList(CyConst.DEVICES_CYUSB);
            devices.DeviceAttached += new EventHandler(usbDevices_DeviceAttached);
            devices.DeviceRemoved += new EventHandler(usbDevices_DeviceRemoved);
        }

        // 事件更改时调用的方法
        private void UpdateDeviceList()
        {
            DeviceUpdateEvent(devList);
        }

        /* Summary
            Event handler for new device attach
        */
        void usbDevices_DeviceAttached(object sender, EventArgs e)
        {
            USBEventArgs usbEvent = e as USBEventArgs;

            string id = $"{usbEvent.VendorID}-{usbEvent.ProductID}-{usbEvent.SerialNum}";
            UpdateDAQs(id, true);

            RefreshDeviceList();
        }

        /* Summary
            Event handler for device removal
        */
        void usbDevices_DeviceRemoved(object sender, EventArgs e)
        {
            USBEventArgs usbEvent = e as USBEventArgs;
            //curEndpt = null; // reinitialize
            string id = $"{usbEvent.VendorID}-{usbEvent.ProductID}-{usbEvent.SerialNum}";
            UpdateDAQs(id, false);

            RefreshDeviceList();
        }

        private void RefreshDeviceList()
        {
            devices = new USBDeviceList(CyConst.DEVICES_CYUSB);

            devList.Clear();
            foreach (USBDevice dev in devices)
            {
                string s = $"{dev.VendorID}-{dev.ProductID}-{dev.SerialNumber}";
                devList.Add(s);
            }
            UpdateDeviceList();
        }

        private void UpdateLsbDAQDevices()
        {
            devices = new USBDeviceList(CyConst.DEVICES_CYUSB);

            devList.Clear();
            foreach (USBDevice item in devices)
            {
                if (!String.IsNullOrEmpty(item.SerialNumber) && item.SerialNumber.Substring(0, 2) == "FT")
                {
                    string id = $"{item.VendorID}-{item.ProductID}-{item.SerialNumber}";
                    devList.Add(id);
                    daqs.Add(id, CatchDAQConfig(id));
                }
            }
            UpdateDeviceList();
        }

        private void UpdateDAQs(string id, bool isAdd)
        {
            if (daqs.ContainsKey(id))
            {
                if (isAdd)
                {
                    MessageBox.Show($"该设备标识{id}已经存在。");
                    return;
                }
                else
                {
                    daqs.Remove(id);
                }
            }
            else
            {
                if (isAdd)
                {
                    daqs.Add(id, CatchDAQConfig(id));
                }
                else
                {
                    MessageBox.Show("设备不存在。");
                    return;
                }
            }
        }

        public DAQDevice CatchDAQConfig(string id)
        {
            DAQDevice dd = new DAQDevice();
            USBDeviceList devices = new USBDeviceList(CyConst.DEVICES_CYUSB);
            USBDevice dev = null;
            for (int i = 0; i < devices.Count; i++)
            {
                string s = $"{devices[i].VendorID}-{devices[i].ProductID}-{devices[i].SerialNumber}";
                if (s == id)
                {
                    dev = devices[i];
                    break;
                }
            }
            dd.dev = (CyUSBDevice)dev;
            DAQLimit lmt = new DAQLimit();
            if (dev != null && dev.VendorID == 1204 && dev.ProductID == 4100)
            {
                DAQHelper.ADCStopClearBuffer(dd.dev);
                DAQHelper.GetDeviceConfig(dd.dev, out lmt);
                dd.Limit = lmt;
                dd.Table.Init(lmt);
            }
            return dd;
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (MessageBox.Show("确定要退出程序吗？", "提示", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk) == DialogResult.OK)
            {
                e.Cancel = false;  //点击OK 
            }
            else
            {
                e.Cancel = true;
            }
        }

        private void tabForms_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(tabForms.SelectedTab == tabConfig)
            {
                LoginForm login = new LoginForm();
                var ret = login.ShowDialog(this);
                if (ret == DialogResult.Yes)
                {
                    configForm.Visible = true;
                }
                else
                {
                    configForm.Visible = false;
                }
            }
        }
    }
}
